from .infrastructure import Record

class YveTransformer:

    def __init__(self, u_map={}):
        self.umap = u_map

    def transform(self, data):
        from .infrastructure import SuperMap
        SuperMap.rep_area_set = set()
        import pandas as pd
        if isinstance(data, pd.Series):
            data = data.astype(str)
        lines = []
        for line in data:
            lines.append(Record(line).pca_map(self.umap))

        import logging
        if len(SuperMap.rep_area_set) != 0:
            logging.warning("多个城市有" + str(SuperMap.rep_area_set)
             + "区")

        import pandas as pd
        return pd.concat(lines, ignore_index=True)



from .infrastructure import SuperMap


# 代表一个(省，市，区)组成的地点
class Location:

    # 直辖市
    # municipality = ["北京市", "天津市", "上海市", "重庆市"]

    def __init__(self):
        self.province = Province()
        self.city = City()
        self.area = Area()

    def setPlace(self, name, place_type):
        if not hasattr(self, place_type):
            from .exceptions import PlaceTypeNotExistException
            raise PlaceTypeNotExistException(place_type + " 地区类型不存在")
        if getattr(self, place_type).isEmpty():
            getattr(self, place_type).setPlace(name)

    def pca_map(self, umap):
        if self.area.isEmpty():
            self.__city_and_province()
        else:
            if (self.area.name not in SuperMap.rep_areas) or (umap.get(self.area.name)):
                if umap.get(self.area.name):
                    tmp = umap.get(self.area.name)
                else:
                    tmp = SuperMap.area_city_mapper.get(self.area.name)
                if self.city.isEmpty() or self.city.precision == 0:
                    self.city.setPlace(tmp)
                elif self.city.isNotEmpty() and self.city.precision == 1:
                    if not self.area.isBlong(self.city.name) \
                            and umap.get(self.area.name) != self.city.name:
                        self.area.reset()
                self.__city_and_province()
            else:  # 重复区名 代码执行路径
                import logging
                SuperMap.rep_area_set.add(self.area.name)
                if self.city.isNotEmpty():
                    self.__city_and_province()

        if self.city.name.isdigit():
            self.city.reset()

        import pandas as pd
        # 组装成DataFrame
        return pd.DataFrame({"省": [self.province.name], "市": [self.city.name], \
                             "区": [self.area.name]})

    def __city_and_province(self):
        if self.city.isNotEmpty() and self.province.isNotEmpty():
            if not self.city.isBlong(self.province.name):
                if self.city.precision > self.province.precision:
                    self.province.name = self.city.belong
                else:
                    self.city.reset()
        elif self.city.isNotEmpty() and self.province.isEmpty():
            self.province.name = self.city.belong


class Place:

    def __init__(self, name=""):
        self.name = name
        self.precision = 1

    def reset(self):
        self.name = ""

    def isBlong(self, mayBe):
        return self.belong == mayBe

    def isEmpty(self):
        return False if self.name else True

    def isNotEmpty(self):
        return True if self.name else False


class City(Place):

    def __init__(self, name=""):
        super().__init__()

    def __getBlong(self):
        return SuperMap.city_province_mapper.get(self.name)

    def setPlace(self, name):
        self.name, isfilled = SuperMap.fillCity(name)
        if isfilled:  # 如果是需要补充字的，则认为这个匹配准确率比较低
            self.precision = 0
        self.belong = self.__getBlong()


class Province(Place):

    def __init__(self, name=""):
        super().__init__()

    def __getBlong(self):
        return SuperMap.province_country_mapper.get(self.name)

    def setPlace(self, name):
        self.name, isfilled = SuperMap.fillProvince(name)
        if isfilled:  # 如果是需要补充字的，则认为这个匹配准确率比较低
            self.precision = 0
        self.belong = self.__getBlong()


class Area(Place):

    def __init__(self, name=""):
        super().__init__()

    def __getBlong(self):
        return SuperMap.area_city_mapper.get(self.name)

    def setPlace(self, name):
        self.name = name
        self.precision = 1
        self.belong = self.__getBlong()












desc ='''
这个就是一个可以把字符串文本里面的地址解析出来并生成一个DataFrame的小包.
用法：
import yve_help
yve_help.transform(location_list)
注意一定是list啊

Dedicate this package to Yve
'''

myumap = {'南关区': '长春市',
          '南山区': '深圳市',
          '宝山区': '上海市',
          '市辖区': '东莞市',
          '普陀区': '上海市',
          '朝阳区': '北京市',
          '河东区': '天津市',
          '白云区': '广州市',
          '西湖区': '杭州市',
          '铁西区': '沈阳市',
          '龙华区': '深圳市'
          }


def transform(location_strs, umap=myumap):
    from mapper.transformer import ChloeTransformer
    cpca = YveTransformer(umap)
    return cpca.transform(location_strs)

#print(transform(['天津市南京路','北京市海淀区北三环西路满庭芳园','上海闸北区','深圳福田']))



import jieba


class Record:
    def __init__(self, line):
        from .domain import Location
        self.location = Location()
        # 默认选取第一个遇到的省，市或者自治区
        for word in jieba.cut(line):
            # 因为jieba对"上海浦东区"的分词会出现问题，所以这里单独处理
            if word == "上海市浦东新区":
                self.location.setPlace("上海市", SuperMap.CITY)
                self.location.setPlace("浦东新区", SuperMap.AREA)
                continue

            word_type = SuperMap.getType(word)

            if word_type:
                self.location.setPlace(word, word_type)

    def pca_map(self, umap):
        return self.location.pca_map(umap)


class SuperMap:
    from .mappers import area_city_mapper, city_province_mapper, \
        province_country_mapper, rep_areas

    AREA = "area"
    CITY = "city"
    PROVINCE = "province"

    rep_area_set = set()

    @classmethod
    def getType(cls, word):
        if cls.area_city_mapper.get(word):
            return cls.AREA
        if cls.city_province_mapper.get(word):
            return cls.CITY
        if cls.province_country_mapper.get(word):
            return cls.PROVINCE
        return ""

    # 补全区级地区的最后一个字
    @classmethod
    def fillCity(cls, word):
        if word and not word.endswith("市") \
                and not word.endswith("盟") \
                and not word.endswith("地区") \
                and not word.endswith("自治州"):
            return word + "市", True
        return word, False

    # 补全省级地区的最后一个字
    @classmethod
    def fillProvince(cls, word):
        if word and not word.endswith("市") and not word.endswith("省"):
            if cls.province_country_mapper.get(word + "市"):
                return word + "市", True
            if cls.province_country_mapper.get(word + "省"):
                return word + "省", True
        return word, False







class CPCAException(Exception):
    pass

class PlaceTypeNotExistException(CPCAException):
    pass

class InputTypeNotSuportException(CPCAException):
    input_type = \
"""
输入应该为
|省     |市    |区     |
|河北省 |廊坊市 |三河县 |

"""
    pass


from setuptools import setup#, find_packages

LONGDOC = '''
All respect to 'DQinYuan'
这个就是一个可以把字符串文本里面的地址解析出来并生成一个DataFrame的小包.
用法：
import yve_help
yve_help.transform(location_list)
注意一定是list啊

Dedicate this package to Yve and I love her
 '''


requires = ['pandas(>=0.20.0)',
            'jieba(>=0.39)',
            ]

setup(name='yve',
      version='0.0.1',
      description='Chinese Province, City and Area Recognition Utilities',
      long_description=LONGDOC,
      author='Hugh',
      license="MIT",
      classifiers=[
          'Intended Audience :: Developers',
          'License :: OSI Approved :: MIT License',
          'Operating System :: OS Independent',
          'Natural Language :: Chinese (Simplified)',
          'Programming Language :: Python :: 3.6',
          'Topic :: Text Processing',
          'Topic :: Text Processing :: Indexing',
      ],
      keywords='for study',
      packages=['mapper',''],
      package_dir={'mapper': 'mapper','': '.', },
      install_requires=requires)